from typing import Annotated, List
from annotated_types import Len
from pydantic import BaseModel, Field, StrictInt

class Response_Creaters_Item(BaseModel):
    username: str
    modelCount: int | None = None
    link: str

class Response_Creaters_Metadata(BaseModel):
    totalItems: int
    currentPage: int
    pageSize: int
    totalPages: int
    nextPage: str

class Response_Creaters(BaseModel):
    items: List[Response_Creaters_Item]
    metadata: Response_Creaters_Metadata


LimitInt = Annotated[int, Field(strict=True, ge=1, le=200)]
Limit = Annotated[List[LimitInt], Len(1, 1)]
class Creators_API_Opts(BaseModel):
    limit: None | Limit = None # The number of results to be returned per page. This can be a number between 0 and 200. By default, each page will return 20 results. If set to 0, it'll return all the creators
    page: None | Annotated[List[StrictInt], Len(1,1)] = None  # The page from which to start fetching creators
    query: None | Annotated[List[str], Len(1,1)] = None # Search query to filter creators by username


from typing import List, Dict, Any, Optional, Annotated
from annotated_types import Len
from pydantic import BaseModel, Field, Strict, StrictBool, StrictInt
import enum
from .images import Period

class Response_Models_Type(enum.Enum):
    Checkpoint = "Checkpoint"
    TextualInversion = "TextualInversion"
    Hypernetwork = "Hypernetwork"
    AestheticGradient = "AestheticGradient"
    LORA = "LORA"
    Controlnet = "Controlnet"
    Poses = "Poses"
    LoCon = "LoCon"
    DoRA = "DoRA"
    Other = "Other"
    MotionModule = "MotionModule"
    Upscaler = "Upscaler"
    VAE = "VAE"
    Wildcards = "Wildcards"
    Workflows = "Workflows"

class Response_Models_Mode(enum.Enum):
    Archived = "Archived"
    TakenDown = "TakenDown"

class Response_Models_Creator(BaseModel):
    username: str 	# The name of the creator
    image: str | None 	# The url of the creators avatar

class Response_Models_Stats(BaseModel):
    downloadCount: int # The number of downloads the model has
    favoriteCount: int # The number of favorites the model has
    thumbsUpCount: int # The number of thumbs up the model has
    thumbsDownCount: int # The number of thumbs down the model has
    commentCount: int # The number of comments the model has
    ratingCount: int # The number of ratings the model has
    rating: float	# The average rating of the model
    tippedAmountCount: int # The number of tips the model has

class Response_Models_modelVersions_Files_Metadata_fp(enum.Enum):
    fp8 = "fp8"
    fp16 = "fp16"
    fp32 = "fp32"

class Response_Models_modelVersions_Files_Metadata_Size(enum.Enum):
    full = "full"
    pruned = "pruned"

class Response_Models_modelVersions_Files_Metadata_Format(enum.Enum):
    SafeTensor = "SafeTensor"
    PickleTensor = "PickleTensor"
    Other = "Other"
    Diffusers = "Diffusers"

class Response_Models_modelVersions_Files_Metadata(BaseModel):
    fp: Response_Models_modelVersions_Files_Metadata_fp | None = None # The specified floating point for the file
    size: Response_Models_modelVersions_Files_Metadata_Size | None = None # The specified model size for the file
    format: Response_Models_modelVersions_Files_Metadata_Format | None = None 	# The specified model format for the file


class Response_Models_modelVersions_Files_PickleScanResult(enum.Enum):
    Pending = 'Pending'
    Success = 'Success'
    Danger = 'Danger'
    Error = 'Error'
    
class Response_Models_modelVersions_Files_VirusScanResult(enum.Enum):
    Pending = 'Pending'
    Success = 'Success'
    Danger = 'Danger'
    Error = 'Error'

class Response_Models_modelVersions_Files_Hashes(BaseModel):
    # AutoV2: str | None = None
    SHA256: str | None = None
    CRC32: str | None = None
    BLAKE3: str | None = None
    AutoV3: str | None = None

class Response_Models_modelVersions_File(BaseModel):
    name: str # file name
    id: int # file id
    sizeKb: float | None = None # The size of the model file
    metadata: Response_Models_modelVersions_Files_Metadata
    pickleScanResult: Response_Models_modelVersions_Files_PickleScanResult # Status of the pickle scan ('Pending', 'Success', 'Danger', 'Error')
    virusScanResult: Response_Models_modelVersions_Files_VirusScanResult # Status of the virus scan ('Pending', 'Success', 'Danger', 'Error')
    scannedAt: str | None = None # (ISO 8601) The date in which the file was scanned
    hashes: Response_Models_modelVersions_Files_Hashes
    downloadUrl: str # model download url: "https://civitai.com/api/download/models/8387"
    primary: bool | None = None # If the file is the primary file for the model version

class Response_Models_modelVersions_Images_Meta(BaseModel):
    ENSD: str | None = None # stringified number
    Size: str | None = None # resolution string 
    seed: int | None = None
    Score: str | None = None # float string
    steps: int | None = None
    prompt: str | None = None
    sampler: str | None = None
    Eta_DDIM: str | None = Field(default=None, alias="Eta DDIM") # float string
    cfgScale: float | None = None
    resources: List[Any] | None = None
    Model_hash: str | None = Field(default=None, alias="Model hash") # short hash string
    Hires_upscale: str | None = Field(default=None, alias="Hires upscale") # float str
    Hires_upscaler: str | None = Field(default=None, alias="Hires upscaler") # hires upscaler name
    negativePrompt: str | None = None
    Denoising_strength: str | None = Field(default=None, alias="Denoising strength") # float string

class Response_Models_modelVersions_Image(BaseModel):
    url: str 	# The url for the image
    nsfwLevel: int 	# Whether or not the image is NSFW (note: if the model is NSFW, treat all images on the model as NSFW)
    width: int 	# The original width of the image
    height: int 	# The original height of the image
    hash: str 	#The blurhash of the image
    meta: Response_Models_modelVersions_Images_Meta | None = None	# The generation params of the image

class Response_Models_modelVersions_Stats(BaseModel):
    downloadCount: int # The number of downloads the model has
    ratingCount: int # The number of ratings the model has
    rating: float # The average rating of the model

class Response_Models_modelVersions_Model(BaseModel):
    name: str # The name of the model
    type: Response_Models_Type
    nsfw: bool # Whether the model is NSFW or not
    poi: bool # Whether the model is a Point of Interest model

class Response_Models_modelVersion(BaseModel):
    id: int	# The identifier for the model version
    modelId: int | None = None # The model id
    baseModel: str # The base model of the model
    name: str # The name of the model version
    earlyAccessEndsAt: str | None = None # (ISO 8601) The date in which the early access ends
    earlyAccessConfig: Dict[str, Any] | None = None # The early access configuration
    uploadType: str | None = None # The upload type of the model version
    description: str | None = None # The description of the model version (usually a changelog)
    createdAt: str | None = None # (ISO 8601) The date in which the version was created
    updatedAt: str | None = None # (ISO 8601) The date in which the version was updated
    status: str | None = None # The status of the model version
    publishedAt: str # (ISO 8601) The date in which the version was published, Only available for the model version query endpoint
    downloadUrl: str 	# The download url to get the model file for this specific version
    trainedWords: List[str] | None = None 	# The words used to trigger the model
    files: List[Response_Models_modelVersions_File]
    images: List[Response_Models_modelVersions_Image] | None = None
    stats: Response_Models_modelVersions_Stats

class Response_Models_Metadata(BaseModel):
    totalItems: int | None = None 	# The total number of items available
    currentPage: int | None = None 	# The the current page you are at
    pageSize: int | None = None 	# The the size of the batch
    totalPages: int | None = None 	# The total number of pages
    nextPage: str | None = None	# The url to get the next batch of items
    prevPage: str | None = None	# The url to get the previous batch of items

class Response_Model(BaseModel):
    id: int 	# The identifier for the model
    name: str 	# The name of the model
    description: str 	# The description of the model (HTML)
    type: Response_Models_Type | None = None # The model type
    nsfw: bool # Whether the model is NSFW or not
    tags: List[str] # The tags associated with the model
    mode: Response_Models_Mode | None = None 	# The mode in which the model is currently on. If Archived, files field will be empty. If TakenDown, images field will be empty
    creator: Response_Models_Creator | None = None 	# The creator of the model
    stats: Response_Models_Stats
    modelVersions: List[Response_Models_modelVersion]

class Response_Models(BaseModel):
    items: List[Response_Model]
    metadata: Response_Models_Metadata

class Sort(enum.Enum):
    Highest_Rated = 'Highest Rated'
    Most_Downloaded = 'Most Downloaded'
    Newest = 'Newest'
    
class AllowCommercialUse(enum.Enum):
    None_ = 'None'
    Image = 'Image'
    Rent = 'Rent'
    Sell = 'Sell'

LimitInt = Annotated[int, Field(strict=True, ge=1, le=100)]
Limit = Annotated[List[LimitInt], Len(1, 1)]
class Models_API_Opts(BaseModel):
    limit: 	None | Limit = None 	# The number of results to be returned per page. This can be a number between 1 and 100. By default, each page will return 100 results
    page: 	None | Annotated[List[StrictInt], Len(1,1)] = None 	# The page from which to start fetching models
    query: 	None | Annotated[List[str], Len(1,1)] = None 	# Search query to filter models by name
    tag: 	None | Annotated[List[str], Len(1,1)] = None 	# Search query to filter models by tag
    username: 	Optional[str] = None 	# Search query to filter models by user
    types: List[Response_Models_Type] | None = None 	# The type of model you want to filter with. If none is specified, it will return all types
    sort: 	None | Annotated[List[Sort], Len(1,1)] = None 	# The order in which you wish to sort the results
    period: None | Annotated[List[Period], Len(1,1)] = None 	# The time frame in which the models will be sorted
    # rating: 	Optional[int] 	# (Deprecated) The rating you wish to filter the models with. If none is specified, it will return models with any rating
    favorites: None | Annotated[List[StrictBool], Len(1,1)] = None 	# (AUTHED) Filter to favorites of the authenticated user (this requires an API token or session cookie)
    hidden: None | Annotated[List[StrictBool], Len(1,1)] = None 	# (AUTHED) Filter to hidden models of the authenticated user (this requires an API token or session cookie)
    primaryFileOnly: None | Annotated[List[StrictBool], Len(1,1)] = None 	# Only include the primary file for each model (This will use your preferred format options if you use an API token or session cookie)
    allowNoCredit: None | Annotated[List[StrictBool], Len(1,1)] = None 	# Filter to models that require or don't require crediting the creator
    allowDerivatives: None | Annotated[List[StrictBool], Len(1,1)] = None 	# Filter to models that allow or don't allow creating derivatives
    allowDifferentLicenses: None | Annotated[List[StrictBool], Len(1,1)] = None # Filter to models that allow or don't allow derivatives to have a different license
    allowCommercialUse: List[AllowCommercialUse] | None = None 	# Filter to models based on their commercial permissions
    nsfw: None | Annotated[List[StrictBool], Len(1,1)] = None # If false, will return safer images and hide models that don't have safe images
    supportsGeneration: None | Annotated[List[StrictBool], Len(1,1)] = None 	# If true, will return models that support generation


from pydantic import BaseModel
from typing import Optional, List, Dict, Any
from enum import Enum
from .models import Response_Models_Type, Response_Models_Stats, Response_Models_Creator, Response_Models_modelVersion

class AllowCommercialUse(Enum):
    Image = "Image"
    RentCivit = "RentCivit"
    Rent = "Rent"
    Sell = "Sell"

class Response_Model_ById(BaseModel):
    id: int
    name: str
    description: str # html text
    allowNoCredit: bool
    allowCommercialUse: List[AllowCommercialUse]
    allowDerivatives: bool
    allowDifferentLicense: bool
    type: Response_Models_Type
    minor: bool
    poi: bool
    nsfw: bool
    nsfwLevel: int
    # cosmetic: None
    stats: Response_Models_Stats
    creator: Response_Models_Creator | None = None
    tags: List[str]
    modelVersions: List[Response_Models_modelVersion]



import enum
from typing import Annotated, Any, Dict, List, Optional
from annotated_types import Len
from pydantic import BaseModel, Field, StrictInt

class NsfwLevel(enum.Enum):
    None_ = 'None'
    Soft = 'Soft'
    Mature = 'Mature'
    X = 'X'

class Sort(enum.Enum):
    Most_Reactions = "Most Reactions"
    Most_Comments = "Most Comments"
    Newest = "Newest"

class Period(enum.Enum):
    AllTime = "AllTime"
    Day = "Day"
    Week = "Week"
    Month = "Month"
    Year = "Year"

class Response_Images_Stats(BaseModel):
    cryCount: int # The number of cry reactions
    laughCount: int # The number of laugh reactions
    likeCount: int # The number of like reactions
    heartCount: int # The number of heart reactions
    commentCount: int # The number of comment reactions

class Response_Images_Meta_CivitaiResources_Type(enum.Enum):
    checkpoint = "checkpoint"
    lora = "lora"

class Response_Images_Meta_Resources(BaseModel):
    hash: str
    name: str
    type: Response_Images_Meta_CivitaiResources_Type
    weight: float | None = None

class Response_Images_Meta_CivitaiResources(BaseModel):
    type: Response_Images_Meta_CivitaiResources_Type
    weight: Optional[float] = None
    modelVersionId: int
    modelVersionName: str | None = None

class Response_Images_Meta(BaseModel): # this field is a non-structured object that contains additional information about the image, the meta structure will be different for each webui
    Size: str
    seed: int
    vaes: str | None = None
    comfy: str | None = None # the json data str
    Model: str
    steps: int
    prompt: str
    sampler: str
    Version: str # webui version str
    cfgScale: int
    clipSkip: str
    resources: List[Response_Images_Meta_Resources]
    civitaiResources: List[Response_Images_Meta_CivitaiResources] # The generation process used model resource information
    Hires_upscale: str | None = None
    Hires_upscaler: str | None = None
    negativePrompt: str
    Denoising_strength: str | None = None
    ADetailer_model: str | None = Field(default=None, alias="ADetailer model")
    ADetailer_version: str | None = Field(default=None, alias="ADetailer version") # version str
    ADetailer_mask_blur: str | None = Field(default=None, alias="ADetailer mask blur") # float str
    ADetailer_confidence: str | None = Field(default=None, alias="ADetailer confidence") # float str
    ADetailer_dilate_erode: str | None = Field(default=None, alias="ADetailer dilate erode") # float str
    ADetailer_inpaint_padding: str | None = Field(default=None, alias="ADetailer inpaint padding") # float str
    ADetailer_denoising_strength: str | None = Field(default=None, alias="ADetailer denoising strength") # float str
    ADetailer_inpaint_only_masked: str | None = Field(default=None, alias="ADetailer inpaint only masked") # bool str
    #   "meta": {
    #     "Size": "512x768",
    #     "seed": 234871805,
    #     "Model": "Meina",
    #     "steps": 35,
    #     "prompt": "<lora:setsunaTokage_v10:0.6>, green hair, long hair, standing, (ribbed dress), zettai ryouiki, choker, (black eyes), looking at viewer, adjusting hair, hand in own hair, street, grin, sharp teeth, high ponytail, [Style of boku no hero academia]",
    #     "sampler": "DPM++ SDE Karras",
    #     "cfgScale": 7,
    #     "Clip skip": "2",
    #     "Hires upscale": "2",
    #     "Hires upscaler": "4x-AnimeSharp",
    #     "negativePrompt": "(worst quality, low quality, extra digits:1.3), easynegative,",
    #     "Denoising strength": "0.4"
    #   },

class Response_Images_Metadata(BaseModel):
    nextCursor: int | str | None = None  # The id of the first image in the next batch
    currentPage: Optional[int] = None # The the current page you are at (if paging)
    pageSize: Optional[int] = None  # The the size of the batch (if paging)
    nextPage: Optional[str] = None     #The url to get the next batch of items

class Response_Images_Item(BaseModel):
    id: int	# The id of the image
    url: str # The url of the image at it's source resolution
    hash: str # The blurhash of the image
    width: int # The width of the image
    height: int	# The height of the image
    nsfw: bool # If the image has any mature content labels
    nsfwLevel: NsfwLevel # The NSFW level of the image
    createdAt: str # (ISO 8601 format) The date the image was posted
    postId: int # The ID of the post the image belongs to
    stats: Response_Images_Stats
    meta: Dict | None = None # this field is a non-structured object that contains additional information about the image, the meta structure will be different for each webui
    username: Optional[str] # The username of the creator
    baseModel: Optional[str] # The base model of the image

class Response_Images(BaseModel):
    items: List[Response_Images_Item]
    metadata: Response_Images_Metadata

LimitInt = Annotated[int, Field(strict=True, ge=1, le=200)]
Limit = Annotated[List[LimitInt], Len(1, 1)]

class Images_API_Opts(BaseModel):
    limit: None | Limit = None # The number of results to be returned per page. This can be a number between 0 and 200. By default, each page will return 100 results.
    postId: None | Annotated[List[StrictInt], Len(1,1)] = None # The ID of a post to get images from
    modelId: None | Annotated[List[StrictInt], Len(1,1)] = None # The ID of a model to get images from (model gallery)
    modelVersionId: None | Annotated[List[StrictInt], Len(1,1)] = None # The ID of a model version to get images from (model gallery filtered to version)
    username: None | Annotated[List[str], Len(1,1)] = None # Filter to images from a specific user
    nsfw: None | List[NsfwLevel] = None # Filter to images that contain mature content flags or not (undefined returns all)
    sort: None | Annotated[List[Sort], Len(1,1)] = None # The order in which you wish to sort the results
    period: None | Annotated[List[Period], Len(1,1)] = None # The time frame in which the images will be sorted
    page: None | StrictInt = None # The page from which to start fetching creators


import enum
from typing import Any, List, Optional, Coroutine, Dict
import anyio.from_thread
import httpx
from civitai_api.models.creators import Response_Creaters, Creators_API_Opts
from civitai_api.models.images import Images_API_Opts, NsfwLevel, Period, Response_Images
from civitai_api.models.models import Models_API_Opts, Response_Models, Response_Models_Type, Response_Models_modelVersion
from civitai_api.models.model_by_version_id import Response_ModelByVersionId
import anyio
import os
from urllib.parse import urljoin

from civitai_api.models.models_by_id import Response_Model_ById

# API endpoints references: https://github.com/civitai/civitai/wiki/REST-API-Reference

API_URL_V1 = "https://civitai.com/api/v1/"
API_URL_V1_Creators = urljoin(API_URL_V1, "creators") # https://civitai.com/api/v1/creators
API_URL_V1_Images = urljoin(API_URL_V1, "images") # https://civitai.com/api/v1/images
API_URL_V1_Models = urljoin(API_URL_V1, "models") # https://civitai.com/api/v1/models
API_URL_V1_Model_By_Id = urljoin(API_URL_V1, "models/") # "https://civitai.com/api/v1/models/:modelId"
API_URL_ModelVersion_By_VersionId = urljoin(API_URL_V1, "model-versions/")  # "https://civitai.com/api/v1/model-versions/:modelVersionId"
API_URL_ModelVersion_By_Hash = urljoin(API_URL_V1, "model-versions/by-hash/") # "https://civitai.com/api/v1/model-versions/by-hash/:hash"
API_URL_Tags = urljoin(API_URL_V1, "tags") # https://civitai.com/api/v1/tags

# def construct_header_from_dict(params: dict) -> dict:
    # params.pop("self", None)
    # params.pop("httpx_client", None)
    # params.pop("httpx_async_client", None)
    # return {k: str(v) for k, v in params.items() if v is not None}

def construct_query_params_from_dict(params: Dict[str, List[Any]]):
    query_params: Dict[str, List[Any]] = {}

    for k,v in params.items():
        if params[k] == None:
            continue
        query_params[k] = []
        for item in v:
            if isinstance(item, bool):
                query_params[k].append(str(item).lower())
            else:
                query_params[k].append(str(item))
    
    return query_params
    
# def is_running_anyio_loop():
#     try:
#         # Attempt to get the current task
#         anyio.get_current_task()
#         return True  # If successful, an AnyIO event loop is running
#     except RuntimeError:
#         return False  # No running AnyIO event loop

def get_creators_v1(
        httpx_client:httpx.Client,
        opts: Creators_API_Opts | None = None
        ) -> Response_Creaters:
    if opts:
        query_params = construct_query_params_from_dict(opts.model_dump())
    else:
        query_params = None
    response = httpx.get(API_URL_V1_Creators, params=query_params)
    return Response_Creaters(**response.json())

async def async_get_creators_v1(
        httpx_async_client:httpx.AsyncClient,
        opts: Creators_API_Opts | None = None
        ) -> Response_Creaters:
    if opts:
        query_params = construct_query_params_from_dict(opts.model_dump())
    else:
        query_params = None
    response = await httpx_async_client.get(API_URL_V1_Creators, params=query_params)
    return Response_Creaters(**response.json())

def get_images_v1(
        httpx_client:httpx.Client,
        opts: Images_API_Opts | None = None
        ) -> Response_Images:
    if opts:
        query_params = construct_query_params_from_dict(opts.model_dump())
    else:
        query_params = None
    response = httpx_client.get(API_URL_V1_Images, params=query_params)
    return Response_Images(**response.json())

async def async_get_images_v1(
        httpx_async_client:httpx.AsyncClient,
        opts: Images_API_Opts | None = None
) -> Response_Images:
    if opts:
        query_params = construct_query_params_from_dict(opts.model_dump())
    else:
        query_params = None
    response = await httpx_async_client.get(API_URL_V1_Images, params=query_params)
    return Response_Images(**response.json())

def get_models_v1(
        httpx_client:httpx.Client,
        opts: Models_API_Opts | None = None
) -> Response_Models:
    if (opts):
        if (opts.favorites or opts.hidden):
            if not (httpx_client.headers):
                raise ValueError("If the \'favorites\' and the \'hidden\' params were set, The api_key must be provided while initialize CivitaiAPI class.")
            
    if opts:
        query_params = construct_query_params_from_dict(opts.model_dump())
    else:
        query_params = None
    response = httpx_client.get(API_URL_V1_Models, params=query_params)
    return Response_Models(**response.json())

async def async_get_models_v1(
        httpx_async_client:httpx.AsyncClient,
        opts: Models_API_Opts | None = None
) -> Response_Models:
    if (opts):
        if (opts.favorites or opts.hidden):
            if not (httpx_async_client.headers):
                raise ValueError("If the \'favorites\' and the \'hidden\' params were set, The api_key must be provided while initialize CivitaiAPI class.")
            
    if opts:
        query_params = construct_query_params_from_dict(opts.model_dump())
    else:
        query_params = None
    response = await httpx_async_client.get(API_URL_V1_Models, params=query_params)
    return Response_Models(**response.json())

def get_model_by_id_v1(
        httpx_client:httpx.Client,
        modelId: int
) -> Response_Model_ById:
    response = httpx_client.get(urljoin(API_URL_V1_Model_By_Id, str(modelId)))
    return Response_Model_ById(**response.json())

async def async_get_model_by_id_v1(
        httpx_async_client:httpx.AsyncClient,
        modelId: int
) -> Response_Model_ById:
    response = await httpx_async_client.get(urljoin(API_URL_V1_Model_By_Id, str(modelId)))
    return Response_Model_ById(**response.json())

def get_model_by_versionId_v1(
        httpx_client:httpx.Client,
        modelVersionId: int
) -> Response_Models_modelVersion:
    response = httpx_client.get(urljoin(API_URL_ModelVersion_By_VersionId, str(modelVersionId)))
    return Response_Models_modelVersion(**response.json())

async def async_get_model_by_versionId_v1(
        httpx_async_client:httpx.AsyncClient,
        modelVersionId: int
) -> Response_Models_modelVersion:
    response = await httpx_async_client.get(urljoin(API_URL_ModelVersion_By_VersionId, str(modelVersionId)))
    return Response_Models_modelVersion(**response.json())

def get_model_by_hash_v1(
        httpx_client:httpx.Client,
        hash: str
) -> Response_Models_modelVersion:
    response = httpx_client.get(urljoin(API_URL_ModelVersion_By_Hash, hash))
    return Response_Models_modelVersion(**response.json())

async def async_get_model_by_hash_v1(
        httpx_async_client:httpx.AsyncClient,
        hash: str
) -> Response_Models_modelVersion:
    response = await httpx_async_client.get(urljoin(API_URL_ModelVersion_By_Hash, hash))
    return Response_Models_modelVersion(**response.json())

class CivitaiAPI:
    def __init__(self, api_key: str, proxy: str | None = None):
        self.api_key = api_key
        self.proxy = proxy
        self.httpx_client = httpx.Client(headers={"Authorization": f"Bearer {api_key}"}, proxy = proxy)
        self.httpx_async_client = httpx.AsyncClient(headers={"Authorization": f"Bearer {api_key}"}, proxy = proxy)

    def get_creators_v1(self, opts: Creators_API_Opts | None = None) -> Response_Creaters:
        return get_creators_v1(self.httpx_client, opts)
    
    async def async_get_creators_v1(self, opts: Creators_API_Opts | None = None) -> Response_Creaters:
        return await async_get_creators_v1(self.httpx_async_client, opts)
    
    def get_images_v1(self, opts: Images_API_Opts | None = None) -> Response_Images:
        return get_images_v1(self.httpx_client, opts)
    
    async def async_get_images_v1(self, opts: Images_API_Opts | None = None) -> Response_Images:
        return await async_get_images_v1(self.httpx_async_client, opts)
    
    def get_models_v1(self, opts: Models_API_Opts | None = None) -> Response_Models:
        return get_models_v1(self.httpx_client, opts)
    
    async def async_get_models_v1(self, opts: Models_API_Opts | None = None) -> Response_Models:
        return await async_get_models_v1(self.httpx_async_client, opts)
    
    def get_model_by_id_v1(self, modelId: int) -> Response_Model_ById:
        return get_model_by_id_v1(self.httpx_client, modelId)
    
    async def async_get_model_by_id_v1(self, modelId: int) -> Response_Model_ById:
        return await async_get_model_by_id_v1(self.httpx_async_client, modelId)
    
    def get_model_by_versionId_v1(self, modelVersionId: int) -> Response_Models_modelVersion:
        return get_model_by_versionId_v1(self.httpx_client, modelVersionId)
    
    async def async_get_model_by_versionId_v1(self, modelVersionId: int) -> Response_Models_modelVersion:
        return await async_get_model_by_versionId_v1(self.httpx_async_client, modelVersionId)
    
    def get_model_by_hash_v1(self, hash: str) -> Response_Models_modelVersion:
        return get_model_by_hash_v1(self.httpx_client, hash)
    
    async def async_get_model_by_hash_v1(self, hash: str) -> Response_Models_modelVersion:
        return await async_get_model_by_hash_v1(self.httpx_async_client, hash)
    


from dotenv import load_dotenv
from src.civitai_api import api
from src.civitai_api.models.images import Images_API_Opts, NsfwLevel
import pytest
import httpx
import anyio
import os
pytestmark = pytest.mark.anyio

load_dotenv()

proxy = os.environ.get('PROXY', None)
api_key = os.environ.get('API_KEY', None)
print(f"Proxy: {proxy}")
print(f"API Key: {api_key}")

@pytest.fixture
def httpx_client():
    with httpx.Client(headers={"Authorization": f"Bearer {api_key}"}, proxy = proxy) as client:
        yield client

@pytest.fixture
async def httpx_async_client():
    async with httpx.AsyncClient(headers={"Authorization": f"Bearer {api_key}"}, proxy = proxy) as async_client:
        yield async_client

class TestAPI_V1:
    def test_get_creators_v1(self, httpx_client):
        response = api.get_creators_v1(httpx_client)
    
    def test_get_images_v1(self, httpx_client):
        # response = api.get_images_v1(httpx_client)
        opts: Images_API_Opts = Images_API_Opts(limit=[1], nsfw=[NsfwLevel.X.value], postId=[11059742])
        response = api.get_images_v1(httpx_client,opts=opts) # when an argument takes an enum type as input, you must use it's .value property, otherwise the query paramter's string value will have a enum class name as prefix. (which isn't a valid input for API endpoint.) 
    
    def test_get_models_v1(self, httpx_client):
        response = api.get_models_v1(httpx_client)

    def test_get_model_byId_v1(self, httpx_client):
        models = api.get_models_v1(httpx_client)

        # for model in models.items:
        #     response = api.get_model_by_id_v1(httpx_client, model.id)

        response = api.get_model_by_id_v1(httpx_client, models.items[0].id)

    def test_get_model_by_versionId_v1(self, httpx_client):
        models = api.get_models_v1(httpx_client)

        # for model in models.items:
        #     for version in model.modelVersions:
        #         response = api.get_model_by_versionId_v1(httpx_client, version.id)

        version_id = models.items[0].modelVersions[0].id
        response = api.get_model_by_versionId_v1(httpx_client, version_id)
        
    def test_get_model_by_hash_v1(self, httpx_client):
        models = api.get_models_v1(httpx_client)

        # for model in models.items:
        #     for version in model.modelVersions:
        #         for file in version.files:
        #             hash = file.hashes.BLAKE3
        #             response = api.get_model_by_hash_v1(httpx_client, hash)

        hash = models.items[0].modelVersions[0].files[0].hashes.BLAKE3
        response = api.get_model_by_hash_v1(httpx_client, hash)

class TestAPI_V1_Async:
    async def test_async_get_creators_v1(self, httpx_async_client):
        response = await api.async_get_creators_v1(httpx_async_client)

    async def test_async_get_images_v1(self, httpx_async_client):
        response = await api.async_get_images_v1(httpx_async_client)

    async def test_async_get_models_v1(self, httpx_async_client):
        response = await api.async_get_models_v1(httpx_async_client)

    async def test_async_get_model_byId_v1(self, httpx_async_client):
        models = await api.async_get_models_v1(httpx_async_client)

        # for model in models.items:
        #     response = await api.async_get_model_by_id_v1(httpx_async_client, model.id)

        response = await api.async_get_model_by_id_v1(httpx_async_client, models.items[0].id)

    async def test_async_get_model_by_versionId_v1(self, httpx_async_client):
        models = await api.async_get_models_v1(httpx_async_client)

        # for model in models.items:
        #     for version in model.modelVersions:
        #         response = await api.async_get_model_by_versionId_v1(httpx_async_client, version.id)

        version_id = models.items[0].modelVersions[0].id
        response = await api.async_get_model_by_versionId_v1(httpx_async_client, version_id)

    async def test_async_get_model_by_hash_v1(self, httpx_async_client):
        models = await api.async_get_models_v1(httpx_async_client)

        # for model in models.items:
        #     for version in model.modelVersions:
        #         for file in version.files:
        #             hash = file.hashes.BLAKE3
        #             response = await api.async_get_model_by_hash_v1(httpx_async_client, hash)

        hash = models.items[0].modelVersions[0].files[0].hashes.BLAKE3
        response = await api.async_get_model_by_hash_v1(httpx_async_client, hash)


from typing import List
from pydantic import BaseModel

class Response_Tags_Metadata(BaseModel):
    totalItems: int # The total number of items available
    currentPage: int # The the current page you are at
    pageSize: int # The the size of the batch
    totalPages: int # The total number of pages
    nextPage: str | None # The url to get the next batch of items
    prevPage: str | None # The url to get the previous batch of items

class Response_Tags_Items(BaseModel):
    name: str # The name of the tag
    modelCount: int # The amount of models linked to this tag
    link: str # Url to get all models from this tag

class Response_Tags(BaseModel):
    items: List[Response_Tags_Items]
    metadata: Response_Tags_Metadata



from typing import List
from pydantic import BaseModel
import enum
from .models import Response_Models_Type

class Response_ModelByVersionId_Model_Mode(enum.Enum):
    Archived = "Archived"
    TakenDown = "TakenDown"

# class Response_ModelByVersionId_Model_Type(enum.Enum):
#     Checkpoint = "Checkpoint"
#     TextualInversion = "TextualInversion"
#     Hypernetwork = "Hypernetwork"
#     AestheticGradient = "AestheticGradient"
#     LORA = "LORA"
#     Controlnet = "Controlnet"
#     Poses = "Poses"

class Response_ModelByVersionId_Model(BaseModel):
    name: 	str 	# The name of the model
    type: 	Response_Models_Type 	# The model type
    nsfw: 	bool 	# Whether the model is NSFW or not
    poi: 	bool 	# Whether the model is of a person of interest or not
    mode: 	Response_ModelByVersionId_Model_Mode | None = None # The mode in which the model is currently on. If Archived, files field will be empty. If TakenDown, images field will be empty

class Response_ModelByVersionId_Files_Metadata_Fp(enum.Enum):
    fp16 = "fp16"
    fp32 = "fp32"

class Response_ModelByVersionId_Files_Metadata_Size(enum.Enum):
    full = "full"
    pruned = "pruned"
    
class Response_ModelByVersionId_Files_Metadata_Format(enum.Enum):
    SafeTensor = "SafeTensor"
    PickleTensor = "PickleTensor"
    Other = "Other"

class Response_ModelByVersionId_Files_Metadata(BaseModel):
    fp: Response_ModelByVersionId_Files_Metadata_Fp | None 	# The specified floating point for the file
    size: Response_ModelByVersionId_Files_Metadata_Size | None 	# The specified model size for the file
    format: Response_ModelByVersionId_Files_Metadata_Format | None 	# The specified model format for the file

class Response_ModelByVersionId_Files(BaseModel):
    sizeKb: 	int 	# The size of the model file
    pickleScanResult: 	str 	# Status of the pickle scan ('Pending', 'Success', 'Danger', 'Error')
    virusScanResult: 	str 	# Status of the virus scan ('Pending', 'Success', 'Danger', 'Error')
    scannedAt: 	str | None 	# (ISO 8601) The date in which the file was scanned
    metadata: Response_ModelByVersionId_Files_Metadata
    

from .models import Response_Models_modelVersions_Stats, Response_Models_modelVersions_Image, Response_Models_modelVersions_File

class Response_ModelByVersionId(BaseModel):
    id: 	int 	# The identifier for the model version
    name: 	str 	# The name of the model version
    description: 	str 	# The description of the model version (usually a changelog)
    baseModel: Response_ModelByVersionId_Model
    modelId: 	int 	# The identifier for the model
    createdAt: 	str 	# (ISO 8601) The date in which the version was created
    updatedAt: 	str 	# (ISO 8601) The date in which the version was updated
    downloadUrl: 	str 	# The download url to get the model file for this specific version
    trainedWords: 	List[str] 	# The words used to trigger the model
    files: Response_Models_modelVersions_File
    stats: Response_Models_modelVersions_Stats
    images: Response_Models_modelVersions_Image
    